/*
 * Copyright (c) 1997-1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine" and 
 *    "Java Apache Project" must not be used to endorse or promote products 
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without 
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *    
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

/*****************************************************************************
 * Description: jserv_mmap.c is used by jserv_balance.c to share server's    *
 *              state and other variables.                                   *
 *              This avoids sending requests to one JServ if another         *
 *              process found it was not present. (fail-over part)           *
 * Comments:    MT-unsafe                                                    *
 * Author:      Jean-Luc Rochat <jlrochat@jnix.com>                          *
 * Version:     $Revision: 1.10 $                                             *
 *****************************************************************************/

#include "jserv.h"

#ifdef _POSIX_MAPPED_FILES
/* ========================================================================= */
/* we are on a system with the mmapp POSIX system call.                      */
/* mmap'ed file is a shared memory read/write by all processes.              */
/* ========================================================================= */
#include <sys/mman.h>
/*****************************************************************************
 * Shared variables                                                          *
 * These variables are pointers on common memory                             *
 *****************************************************************************/
caddr_t shmf;
struct shm_mmaped_file * _shmf;
struct shared_hdr * _hdr;
struct shared_host * _host;

struct stat filestat;
void jserv_dbgshm(jserv_config *cfg);

/* ========================================================================= */
/* We overwrite(or just fill) the file and initialize datas                  */
/* ========================================================================= */
int create_shmfile (jserv_config *cfg, int fd) {
        int i, nb;
	char *ptr;
	struct shared_hdr h;
	struct shared_host js[NB_MAX_JSERVS];

	for  (ptr=(char *)&h, i=0; i< sizeof(h); i++)
		ptr[i]='$';
	for  (ptr=(char *)&js, i=0; i< sizeof(js); i++)
		ptr[i]='$';
        jserv_error(JSERV_LOG_DEBUG,cfg,"jserv_mmap:writing ");
	
	strcpy (h.ident, "jserv_mmap");
	h.watchdog_pid = 0;
	h.nb = 0;
        if (nb = write (fd, &h, sizeof(h)) !=  sizeof(h)) {
	    /* error */
            jserv_error(JSERV_LOG_EMERG,cfg,"jserv_mmap:error writing ");
        }
	for (i=0;i<NB_MAX_JSERVS;i++) {
	    js[i].state = DOWN;
	    js[i].name[0] = '\0';
	}
        if (nb = write (fd, js, sizeof(js)) !=  sizeof(js)) {
	    /* error */
            jserv_error(JSERV_LOG_EMERG,cfg,"jserv_mmap:error writing ");
        }
}
        
	

/* ========================================================================= */
/* unmmaps a file                                                            */ 
/* ========================================================================= */
void munmapjservfile()
{
    if (shmf) {
        munmap((caddr_t)shmf, filestat.st_size);
        shmf = (caddr_t) 0;
        _shmf = (struct shm_mmaped_file *) 0;
    }
}

/* ========================================================================= */
/* mmaps a file                                                              */ 
/* ========================================================================= */


struct shm_mmaped_file *  mmapjservfile(jserv_config *cfg, char *filename) {
    int fd;

    if (_shmf) {
        jserv_error(JSERV_LOG_DEBUG,cfg,"jserv_mmap:(%d) remmaping", getpid());
        return _shmf;
	/*
        munmapjservfile();
        shmf = 0;
        _shmf = 0;
        _host = 0;
        _hdr = 0;
        */
    }
   
    jserv_error(JSERV_LOG_DEBUG,cfg,"jserv_mmap:(%d) open %s ", getpid(), filename);

    ap_block_alarms();
    fd = open(filename, O_RDWR|O_CREAT, 0777);
    if (fd == -1) {
        jserv_error(JSERV_LOG_EMERG,cfg,"jserv_mmap: cant open %s errno=%d", filename, errno);
        return NULL;
    }
    if (stat(filename, &filestat) == -1) {
        jserv_error(JSERV_LOG_EMERG,cfg,"jserv_mmap: unable to stat %s", filename);
        return NULL;
    }
    if (filestat.st_size == 0) {
	create_shmfile (cfg, fd);
        if (stat(filename, &filestat) == -1) {
            jserv_error(JSERV_LOG_EMERG,cfg,"jserv_mmap: unable to stat %s", filename);
	}
    }
    shmf = mmap(NULL, filestat.st_size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
    if (shmf == (caddr_t)-1) {
        jserv_error(JSERV_LOG_EMERG,cfg,"jserv_mmap: can't mmap %s errno=%d", filename, errno);
	close(fd);
	ap_unblock_alarms();         
        return NULL;
    }
    close(fd);
    ap_unblock_alarms();

    if (filestat.st_size < sizeof(struct shared_hdr)+ (NB_MAX_JSERVS*sizeof (struct shared_host))) {
        jserv_error(JSERV_LOG_EMERG,cfg,"jserv_mmap: file %s sz is too small corrupted ?", filename);
        return NULL;
    } 
    if (shmf && (strcmp(shmf, "jserv_mmap"))) {
        jserv_error(JSERV_LOG_EMERG,cfg,"jserv_mmap: file %s is corrupted", filename);
        return NULL;
    } 
    _shmf = (struct shm_mmaped_file *) shmf;
    _hdr = (struct shared_hdr *) &_shmf->hdr;
    _host = (struct shared_host *)_shmf->hostarray;
    return _shmf;

}

/* Here we map a structure over the mmaped file */


struct shared_host * jserv_addhost(jserv_config *cfg, jserv_host *cur) {
	struct shared_host *host;
	int nbcurr = 0;
	host = (struct shared_host *)_host;

        nbcurr = _hdr->nb++; 
        host = (struct shared_host *) &_host[nbcurr];
        
	host->state = DOWN;
	strncpy (host->name, cur->id, sizeof(host->name));
	return host;
}

struct shared_host * jserv_gethost(jserv_config *cfg, jserv_host *cur) {
	struct shared_host *host;
	int nbcurr;
	host = (struct shared_host *)_host;
	nbcurr = 0;

	while  (nbcurr < _hdr->nb ) {
   	    nbcurr++;

  	    switch  (host->state) {
		case DOWN:
		case SHUTDOWN_IMMEDIATE: 
		case SHUTDOWN_GRACEFUL: 
		case UP: 
			break;
		default :
			return (struct shared_host *) 0; 
			break;
            }

	    if (!strcmp(host->name, cur->id))
		return (host);
	    host++;
	}
	return (struct shared_host *) 0; 
}

int jserv_hststatus(jserv_config *cfg, jserv_host *cur, char * status) {
	struct shared_host  * host;
	*status = '\0';
	jserv_error(JSERV_LOG_DEBUG,cfg,"jserv_mmap:(%d) (isdead  %s)", getpid(),cur->id);
	if (!shmf)
 	   return -1;
	host = jserv_gethost(cfg, cur);
	if (host == 0)
            return -1;
	*status = host->state;	
	return 0;	
}
int jserv_isup(jserv_config *cfg, jserv_host *cur) {
	struct shared_host  * host;
	if (!shmf)
 	   return 1;
	host = jserv_gethost(cfg, cur);
	if (host == 0)
            return 1;
	return (host->state == UP);	
}
int jserv_isdead(jserv_config *cfg, jserv_host *cur) {
	struct shared_host  * host;
	if (!shmf)
 	   return 0;
	host = jserv_gethost(cfg, cur);
	if (host == 0)
            return 0;
	return (host->state == DOWN);	
}

void set_UP (jserv_config *cfg, struct shared_host  *host) {
	host->state = UP;
}
int is_UP(jserv_config *cfg, struct shared_host  *host) {
	return (host->state == UP);
}


void jserv_set_DOWN(jserv_config *cfg, struct shared_host  *host) {
	host->state = DOWN;
}
int is_DOWN(jserv_config *cfg, struct shared_host  *host) {
	return (host->state == DOWN);
}


void jserv_set_SHUTDOWN_GRACEFUL(jserv_config *cfg, struct shared_host  *host) {
	host->state = SHUTDOWN_GRACEFUL;
}
int is_SHUTDOWN_GRACEFUL(jserv_config *cfg, struct shared_host  *host) {
	return (host->state == SHUTDOWN_GRACEFUL);
}


void jserv_set_SHUTDOWN_IMMEDIATE(jserv_config *cfg, struct shared_host  *host) {
	host->state = SHUTDOWN_IMMEDIATE;
}
int is_SHUTDOWN_IMMEDIATE(jserv_config *cfg, struct shared_host  *host) {
	return (host->state == SHUTDOWN_IMMEDIATE);
}


void jserv_setalive(jserv_config *cfg, jserv_host *cur) {
	struct shared_host  * host;
	if (!shmf)
 	   return; 
	host = jserv_gethost(cfg, cur);
	if (host == 0)
		host = jserv_addhost(cfg, cur);
	if (host == 0)
            return;
	if (host->state == DOWN) {
		/* don't overwrite SHUTDOWN admin cmds */
		host->state = UP;
	}
}
void jserv_setdead(jserv_config *cfg, jserv_host *cur) {
	struct shared_host  * host;
	jserv_dbgshm(cfg);
	if (!shmf)
 	   return; 
	host = jserv_gethost(cfg, cur);
	if (host == 0) {
    		jserv_error(JSERV_LOG_DEBUG,cfg,"jserv_mmap:(%d) adding", getpid());
		host = jserv_addhost(cfg, cur);
	}
	if (host == 0)
            return;
	host->state = DOWN;
	host->hostaddr = cur->hostaddr;
	host->port = cur->port;
	/*jserv_dbgshm(cfg);*/
}
void jserv_dbgshm(jserv_config *cfg) {
	struct shared_host *host;
	int nbcurr;
	host = (struct shared_host *)_host;
	nbcurr = 0;

    	jserv_error(JSERV_LOG_DEBUG,cfg,"jserv_mmap:(%d) ---------------", getpid());
	if (!_host) {
    		jserv_error(JSERV_LOG_DEBUG,cfg,"jserv_mmap:(%d) _host is null", getpid());
		return;
	} 

	while  (nbcurr++ < _hdr->nb ) {
	    switch  (host->state) {
		case DOWN:
		case SHUTDOWN_IMMEDIATE: 
		case SHUTDOWN_GRACEFUL: 
		case UP: 
    			jserv_error(JSERV_LOG_DEBUG,cfg,"jserv_mmap:(%d) state = %c  name = %s", getpid(), host->state, host->name);
			break;
		default :
    			jserv_error(JSERV_LOG_DEBUG,cfg,"jserv_mmap:(%d) state = %c file corrupted", getpid(), host->state);
			return;
	    }
	    host++;
	}
	return;
}	

#else

/* ========================================================================= */
/* we are on a system without the mmapp POSIX system call.                   */
/* these datas won't be shared by other processes.                           */
/* ========================================================================= */

struct shm_mmaped_file *  mmapjservfile(jserv_config *cfg, char *filename) {
        return  0;
}

void munmapjservfile() { 
}

int jserv_isup(jserv_config *cfg, jserv_host *cur) {
	return 1;
}

int jserv_isdead(jserv_config *cfg, jserv_host *cur) {
	return 0;
}

void jserv_setalive(jserv_config *cfg, jserv_host *cur) {
}

void jserv_setdead(jserv_config *cfg, jserv_host *cur){
}

void jserv_dbgshm(jserv_config *cfg) {
}
#endif

